
// Configuration: list of node ids to not persist as expanded
var collapsedNodes = [];

dojo.addOnLoad(function() {

    dijit._Widget._blankGif = "blank.gif";

    var store = new dojox.data.JsonRestStore({
        target : "servers/"
    });

    var model = new dijit.tree.TreeStoreModel({
        query : "",
        store : store

    });

    // Override to support erroneous children (e.g. 403)
    model.getChildren = function(/* dojo.data.Item */ parentItem,
                                 /* function(items) */ onComplete, 
                                 /* function */ onError) {

        // summary:
        // Calls onComplete() with array of child items of given parent item,
        // all loaded.

        var store = this.store;

        // get children of specified item
        var childItems = [];
        for ( var i = 0; i < this.childrenAttrs.length; i++) {
            var vals = store.getValues(parentItem, this.childrenAttrs[i]);
            childItems = childItems.concat(vals);
        }

        // count how many items need to be loaded
        var _waitCount = 0;
        dojo.forEach(childItems, function(item) {
            if (!store.isItemLoaded(item)) {
                _waitCount++;
            }
        });

        if (_waitCount == 0) {
            // all items are already loaded. proceed...
            onComplete(childItems);
        } else {
            // still waiting for some or all of the items to load
            var onItem = function onItem(item) {
                if (--_waitCount == 0) {
                    // all nodes have been loaded, send them to the tree
                    onComplete(childItems);
                }
            }
            dojo.forEach(childItems, function(item) {
                if (!store.isItemLoaded(item)) {
                    store.loadItem({
                        item : item,
                        onItem : onItem,
                        onError : onItem
                    });
                }
            });
        }
    }

    var tree = new dijit.Tree(
            {
                model : model,
                persist : true,
                collapsedNodes: collapsedNodes,
                onClick : function(item, node) {
                    grid.startup();
                    if (item.__id == "servers/servers/#0") {
                        dojo.byId("path").innerHTML = "servers/";
                        dojo.byId("path").href = "servers/";
                    } else {
                        dojo.byId("path").innerHTML = item.__id.replace(/ /g,
                                '&nbsp;');
                        dojo.byId("path").href = item.__id;
                    }

                    if (item.__id.indexOf("attributes") == -1
                            && item.__id.indexOf("operations") == -1) {
                        hideAll();
                        grid.setStore(new dojo.data.ItemFileReadStore({
                            data : {
                                items : []
                            }
                        }));
                        return;
                    }

                    if (item.children && item.children instanceof Array) {

                        // the node is "attributes" or "operations"

                        // the 'onComplete' method defines what to do when all
                        // children have been loaded
                        var onComplete = function onComplete(items) {
                            var gridItems = [];
                            for (i = 0; i < items.length; i++) {
                                item = items[i];
                                gridItems.push({
                                    value : item.value,
                                    label : item.label,
                                    __id : item.__id
                                });
                            };

                            gridItems.sort(function(x, y) {
                                return ((x.label < y.label) ? -1
                                        : ((x.label > y.label) ? 1 : 0));
                            });
                            var gridStore = new dojo.data.ItemFileReadStore({
                                data : {
                                    items : gridItems
                                }
                            });
                            grid.setStore(gridStore);
                        }

                        model.getChildren(item, onComplete, model.onError);

                        showGrid();
                        grid.render(); // avoids collapse of the columns

                    } else {
                        
                        // the node is a single "attribute" or "operation"
                        
                        document.getElementById("frame").src = item.__id
                                + "?skin=embedded";
                        showFrame();
                    }
                },
                onDblClick : function(item, node) {
                    window.open(item.__id);
                },
                onOpen : function(item, node) {
                    var children = node.getChildren();
                    if (children && children.length == 1) {
                        var node = children[0];
                        if (node.isExpandable && !node.isExpanded) {
                            dijit.byId('tree')._expandNode(node);
                        }
                    }
                },
                getIconClass : function(/* dojo.data.Item */item, /* Boolean */
                        opened) {
                    if (item.__id == "servers/servers/#0") {
                        return "servers";
                    }

                    var nodetype = this.model.store.getValue(item, "nodetype");
                    if (nodetype == null) {
                        return "undefined";
                    }

                    return nodetype;
                },
                
                _onNodeMouseEnter: function(node,event) {
                    if (node.isExpanded == false) {
                        if (this.model.store.getValue(node.item, "tooltip")) {
                            dijit.showTooltip(this.model.store.getValue(node.item, "tooltip"), node.labelNode);
                        }
                    }
                },
                _onNodeMouseLeave: function (node,evt) {
                    dijit.hideTooltip(node.labelNode);
                },
                _saveState: function(){
                    // Create and save a cookie with the currently expanded nodes identifiers
                    // Overre the default saveState function, so we can filter the node identifies that
                    // should not be persisted as expanded
                    if(!this.persist){
                        return;
                    }
                    var ary = [];
                    for(var id in this._openedItemIds){
                        var collapsed = false
                        for (var i = 0; i < this.collapsedNodes.length; i++) {
                            var collapsedNode = this.collapsedNodes[i];
                            if (id.indexOf(collapsedNode) != -1) {
                                collapsed = true;
                            }
                        }
                        if (!collapsed) {
                            ary.push(id);
                        }
                    }
                    dojo.cookie(this.cookieName, ary.join(","), {expires:365});
                },
                
            }, "tree");

    // set the layout structure:
    var layout = [ {
        field : 'label',
        name : 'Item',
        width : '50%'
    }, {
        field : 'value',
        name : 'Value',
        width : 'auto'
    } ];

    // create a new grid:
    var grid = new dojox.grid.DataGrid(
            {
                query : {
                    label : '*'
                },
                store : new dojo.data.ItemFileReadStore({
                    data : {
                        items : []
                    }
                }),
                clientSort : false,
                structure : layout,
                height : '100%',
                style : "font-size:8pt;font-family:Tahoma,Helvetica;",
                onRowDblClick : function(event) {
                    window.open(grid.getItem(event.rowIndex).__id);
                },
                onRowClick : function(event) {
                    document.getElementById("frame").src = grid
                            .getItem(event.rowIndex).__id
                            + "?skin=embedded";
                    showFrame();
                }
            }, "grid");

});

function resize_iframe() {

    var height = window.innerWidth;//Firefox
    if (document.body.clientHeight) {
        height = document.body.clientHeight;//IE
    }
    //resize the iframe according to the size of the
    //window (all these should be on the same line)
    document.getElementById("frame").style.height = parseInt(height
            - document.getElementById("frame").offsetTop - 44)
            + "px";
}

// this will resize the iframe every
// time you change the size of the window.
window.onresize = resize_iframe;

function showFrame() {
    document.getElementById("grid").style.display = 'none';
    document.getElementById("frame").style.display = 'block';
    resize_iframe();
}

function showGrid() {
    document.getElementById("frame").style.display = 'none';
    document.getElementById("grid").style.display = 'block';
    document.getElementById("frame").src = "";
}

function hideAll() {
    document.getElementById("frame").style.display = 'none';
    document.getElementById("grid").style.display = 'none';
    document.getElementById("frame").src = "";
}
